"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const get_jsend_1 = require("../helpers/get-jsend");
const get_request_body_1 = require("../helpers/get-request-body");
const check_create_route_request_1 = require("../helpers/check-create-route-request");
/**
 * @api {post} :port/=^.^=/route Create a new route

 * @apiGroup HttpMocker
 * @apiVersion 0.4.0
 * @apiName CreateRoute
 * @apiDescription It creates a new route in HttpMocker
 *
 * @apiParam {Object} filters         The route filters.
 * @apiParam {String} filters.path    The route path.
 * @apiParam {String} filters.method  The route method.
 * @apiParam {Object} [response]        The route response. It musts be set if does not exist a validator.
 * @apiParam {Number} response.code   The route response code. It musts be between 100 and 600.
 * @apiParam {String} response.body   The route response body.
 * @apiParam {Object[]} [validator]   The route request validator(s), which checks the request.
 * @apiParam {Object} validator.matchers   The validator request matcher.
 * @apiParam {String||Object} validator.matchers.body   The validator request body matcher.
 * @apiParam {String[String||Object]} validator.matchers.header   The validator request header matcher. A object map with string keys and string values or objects values.
 * @apiParam {String} validator.code   The code response when validator(s) matched.
 * @apiParam {String||Object} validator.body   The body response when validator(s) matched.
 *
 * @apiSuccess {String} status Request status
 * @apiSuccess {String} message Success message
 *
 * @apiSuccessExample Success-Response:
 *     HTTP/1.1 200 OK
 *     {
 *       "status": "success",
 *       "message": "route successfully created"
 *     }
 *
 * @apiError {String} status Request status.
 * @apiError {String} message Error message.

 * @apiErrorExample Error-Response:
 *     HTTP/1.1 400 Bad Request
 *     {
 *        "status": "fail",
 *        "message": "request with invalid route path"
 *     }
 */
class CreateRouteHandler {
    constructor(mocker) {
        this.mocker = mocker;
    }
    handle(req) {
        return get_request_body_1.default(req).then((body) => {
            let route = JSON.parse(body);
            route.response = route.response;
            let err = check_create_route_request_1.default(route);
            if (err)
                return { code: 400, body: get_jsend_1.default({ statusCode: 400, data: undefined, message: err }) };
            if (route.response)
                route.response.body = typeof route.response.body != 'string' ? JSON.stringify(route.response.body) : route.response.body;
            if (route.validator)
                for (let i = 0; i < route.validator.length; i++)
                    if (route.validator && route.validator[i].matchers.body)
                        route.validator[i].matchers.body = typeof route.validator[i].matchers.body != 'string' ? JSON.stringify(route.validator[i].matchers.body) : route.validator[i].matchers.body;
            if (this.mocker.getRouteShelf().setItem(route)) {
                return {
                    code: 200,
                    body: get_jsend_1.default({ statusCode: 200, data: undefined, message: 'route successfully created' })
                };
            }
            else {
                return {
                    code: 400,
                    body: get_jsend_1.default({ statusCode: 400, data: undefined, message: 'route already created in this mocker' })
                };
            }
        }).catch((err) => {
            return {
                code: 400,
                body: get_jsend_1.default({ statusCode: 400, data: undefined, message: 'request missing body. ' + err.message })
            };
        });
    }
}
exports.default = CreateRouteHandler;
