"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const get_jsend_1 = require("../helpers/get-jsend");
const get_request_body_1 = require("../helpers/get-request-body");
const check_create_ws_route_request_1 = require("../helpers/check-create-ws-route-request");
const WebSocket = require("ws");
const kitty_1 = require("../consts/kitty");
/**
 * @api {post} :port/=^.^=/ws-route Create a new websocket route

 * @apiGroup HttpMocker
 * @apiVersion 0.4.0
 * @apiName CreateWSRoute
 * @apiDescription It creates a new websocket route in HttpMocker and automatically a pair called :port/ws-pair/ROUTE_PATH, everything sent in a side will be sent to the other, if there is no connection data will be storage and sent immediately when connected.
 *
 * @apiParam {Object} filters         The route filters.
 * @apiParam {String} filters.path    The route path.
 *
 * @apiSuccess {String} status Request status
 * @apiSuccess {String} message Success message
 *
 * @apiSuccessExample Success-Response:
 *     HTTP/1.1 200 OK
 *     {
 *       "status": "success",
 *       "message": "route successfully created"
 *     }
 *
 * @apiError {String} status Request status.
 * @apiError {String} message Error message.

 * @apiErrorExample Error-Response:
 *     HTTP/1.1 400 Bad Request
 *     {
 *        "status": "fail",
 *        "message": "request with invalid route path"
 *     }
 */
class CreateWsRouteHandler {
    constructor(mocker) {
        this.mocker = mocker;
    }
    handle(req) {
        return get_request_body_1.default(req).then((body) => {
            let route = JSON.parse(body);
            route.response = route.response;
            let err = check_create_ws_route_request_1.default(route.filters, route.response);
            if (err) {
                return { code: 400, body: get_jsend_1.default({ statusCode: 400, data: undefined, message: err }) };
            }
            let ok = this.mocker.getWsShelf().setWs({
                routeWs: {
                    path: route.filters.path,
                    server: new WebSocket.Server({ noServer: true }),
                    socket: undefined,
                    messages: []
                },
                routePairWs: {
                    path: `/${kitty_1.RESERVED_PATH}/ws-pair${route.filters.path}`,
                    server: new WebSocket.Server({ noServer: true }),
                    socket: undefined,
                    messages: []
                }
            });
            if (ok) {
                return {
                    code: 200,
                    body: get_jsend_1.default({ statusCode: 200, data: undefined, message: 'route successfully created' })
                };
            }
            else {
                return {
                    code: 400,
                    body: get_jsend_1.default({ statusCode: 400, data: undefined, message: 'route already created in this mocker' })
                };
            }
        }).catch((err) => {
            return {
                code: 400,
                body: get_jsend_1.default({ statusCode: 400, data: undefined, message: 'request missing body. ' + err.message })
            };
        });
    }
}
exports.default = CreateWsRouteHandler;
