"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const get_jsend_1 = require("../helpers/get-jsend");
const get_and_check_query_1 = require("../helpers/get-and-check-query");
/**
 * @api {get} :port/=^.^=/history Get a route history

 * @apiGroup HttpMocker
 * @apiVersion 0.4.0
 * @apiName GetHistory
 * @apiDescription It gets a route history.
 *
 * @apiParam {String} path    The route path. It musts be a query param. If with query, must be encoded.
 * @apiParam {String} method  The route method. It musts be a query param.
 *
 * @apiSuccess {String} status Request status.
 * @apiSuccess {Object} data An array with requests history.
 * @apiSuccess {String} data.ip Remote IP which made the request.
 * @apiSuccess {Object} data.header Request header.
 * @apiSuccess {String} data.header.connection Request connection type.
 * @apiSuccess {String} data.header.contentType Request content type.
 * @apiSuccess {String} data.body Request body.
 * @apiSuccess {String} data.method Request method.
 * @apiSuccess {String} data.url Request path.
 * @apiSuccess {String} data.date Request date.
 * @apiSuccessExample Success-Response:
 *     HTTP/1.1 200 OK
 *     {
 *       "status": "success",
 *       "data": [
 *           {
 *            "ip": "127.0.0.1",
 *            "header": {
 *             "connection": "keep-alive",
 *             "contentType": "application/json"
 *             },
 *            "body": "test",
 *            "method": "GET",
 *            "url": "/oi",
 *            "date": "Wed Jan 29 2020 11:28:02 GMT-0300 (GMT-03:00)"
 *          }
 *       ]
 *    }
 *
 * @apiError {String} status Request status.
 * @apiError {String} message Error message.

 * @apiErrorExample Error-Response:
 *     HTTP/1.1 404 Not Found
 *     {
 *       "status": "fail",
 *       "message": "route does not exist"
 *     }
 */
class HistoryGetterHandler {
    constructor(mocker) {
        this.mocker = mocker;
    }
    handle(req) {
        return new Promise((resolve) => {
            let response = get_and_check_query_1.default(req);
            if (response.err)
                return resolve({
                    code: 400,
                    body: get_jsend_1.default({ statusCode: 400, data: undefined, message: response.err })
                });
            this.mocker.getRouteShelf().getItem(response.query.path, response.query.method)
                .then(() => {
                let requests = this.mocker.getRequestShelf().getRequests(response.query.method.toUpperCase() + response.query.path);
                resolve({
                    code: 200,
                    body: get_jsend_1.default({ statusCode: 200, data: requests, message: undefined })
                });
            })
                .catch((code) => {
                resolve({
                    code: code,
                    body: get_jsend_1.default({ statusCode: code, data: undefined, message: 'route does not exist' })
                });
            });
        });
    }
}
exports.default = HistoryGetterHandler;
