"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const get_jsend_1 = require("../helpers/get-jsend");
const get_and_check_query_1 = require("../helpers/get-and-check-query");
/**
 * @api {get} :port/=^.^=/route Get a route details

 * @apiGroup HttpMocker
 * @apiVersion 0.4.0
 * @apiName GetRoute
 * @apiDescription It gets a route details.
 *
 * @apiParam {String} path    The route path. It musts be a query param. If with query, must be encoded.
 * @apiParam {String} method  The route method. It musts be a query param.
 *
 * @apiSuccess {String} status Request status.
 * @apiSuccess {Object} data An object with route details.
 * @apiSuccessExample Success-Response:
 *     HTTP/1.1 200 OK
 *     {
 *       "status": "success",
 *       "data": {
 *       "filters": {
 *          "path": "/oi",
 *          "method": "GET"
 *     },
 *       "validator": [
 *        {
 *          "matchers": {
 *           "body":"teste"
 *      }
 * },
 *     "code": 200,
 *      "body": "valid"
 *   }
 * }
 *
 * @apiError {String} status Request status.
 * @apiError {String} message Error message.

 * @apiErrorExample Error-Response:
 *     HTTP/1.1 404 Not Found
 *     {
 *       "status": "fail",
 *       "message": "route does not exist"
 *     }
 */
class RouteGetterHandler {
    constructor(mocker) {
        this.mocker = mocker;
    }
    handle(req) {
        return new Promise((resolve) => {
            let response = get_and_check_query_1.default(req);
            if (response.err)
                return resolve({
                    code: 400,
                    body: get_jsend_1.default({ statusCode: 400, data: undefined, message: response.err })
                });
            this.mocker.getRouteShelf().getItem(response.query.path, response.query.method)
                .then((route) => {
                resolve({
                    code: 200,
                    body: get_jsend_1.default({ statusCode: 200, data: route, message: undefined })
                });
            })
                .catch((code) => {
                resolve({
                    code: code,
                    body: get_jsend_1.default({ statusCode: code, data: undefined, message: 'route does not exist' })
                });
            });
        });
    }
}
exports.default = RouteGetterHandler;
